@@ -19,7 +19,7 @@ void       b_buffer_set_fd(b_buffer *buf, int fd);
 size_t     b_buffer_size(b_buffer *buf);
 size_t     b_buffer_unused(b_buffer *buf);
 int        b_buffer_full(b_buffer *buf);
-ssize_t    b_buffer_reclaim(b_buffer *buf, size_t used, size_t given);
+off_t      b_buffer_reclaim(b_buffer *buf, size_t used, size_t given);
 void *     b_buffer_get_block(b_buffer *buf, size_t len, ssize_t *given);
 ssize_t    b_buffer_flush(b_buffer *buf);
 void       b_buffer_reset(b_buffer *buf);
