@@ -38,8 +38,9 @@
 #define B_HEADER_MODE_FORMAT      "%.7o"
 #define B_HEADER_UID_FORMAT       "%.7o"
 #define B_HEADER_GID_FORMAT       "%.7o"
-#define B_HEADER_SIZE_FORMAT     "%.11o"
-#define B_HEADER_MTIME_FORMAT    "%.11o"
+#define B_HEADER_INT_SIZE_FORMAT  "%.11o"
+#define B_HEADER_LONG_SIZE_FORMAT "%.11llo"
+#define B_HEADER_MTIME_FORMAT     "%.11o"
 #define B_HEADER_CHECKSUM_FORMAT  "%.6o"
 #define B_HEADER_MAJOR_FORMAT     "%.7lo"
 #define B_HEADER_MINOR_FORMAT     "%.7lo"
